@@ -18,7 +18,8 @@ from strawberry.experimental.pydantic.conversion import (
 from strawberry.experimental.pydantic.fields import get_basic_type
 from strawberry.experimental.pydantic.utils import (
     DataclassCreationFields,
-    defaults_into_factory,
+    ensure_all_auto_fields_in_pydantic,
+    get_default_factory_for_field,
     get_private_fields,
     sort_creation_fields,
 )
@@ -37,21 +38,21 @@ def replace_pydantic_types(type_: Any):
         # Literal does not have types in its __args__ so we return early
         return type_
     if hasattr(type_, "__args__"):
-        new_type = type_.copy_with(
+        replaced_type = type_.copy_with(
             tuple(replace_pydantic_types(t) for t in type_.__args__)
         )
 
-        if isinstance(new_type, TypeDefinition):
+        if isinstance(replaced_type, TypeDefinition):
             # TODO: Not sure if this is necessary. No coverage in tests
             # TODO: Unnecessary with StrawberryObject
 
-            new_type = builtins.type(
-                new_type.name,
+            replaced_type = builtins.type(
+                replaced_type.name,
                 (),
-                {"_type_definition": new_type},
+                {"_type_definition": replaced_type},
             )
 
-        return new_type
+        return replaced_type
 
     if issubclass(type_, BaseModel):
         if hasattr(type_, "_strawberry_type"):
@@ -111,6 +112,10 @@ def type(
         if not fields_set:
             raise MissingFieldsListError(cls)
 
+        ensure_all_auto_fields_in_pydantic(
+            model=model, auto_fields=fields_set, cls_name=cls.__name__
+        )
+
         all_model_fields: List[DataclassCreationFields] = [
             DataclassCreationFields(
                 name=field_name,
@@ -120,10 +125,7 @@ def type(
                     graphql_name=field.alias if field.has_alias else None,
                     # always unset because we use default_factory instead
                     default=UNSET,
-                    default_factory=defaults_into_factory(
-                        default=field.default if not field.required else UNSET,
-                        default_factory=field.default_factory or UNSET,
-                    ),
+                    default_factory=get_default_factory_for_field(field),
                     type_annotation=get_type_for_field(field),
                     description=field.field_info.description,
                 ),
