@@ -1,15 +1,7 @@
 from __future__ import annotations
 
 from enum import Enum
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Tuple, Type, Union, cast
-
-
-# TypeGuard is only available in typing_extensions => 3.10, we don't want
-# to force updates to the typing_extensions package so we only use it when
-# TYPE_CHECKING is enabled.
-
-if TYPE_CHECKING:
-    from typing_extensions import TypeGuard
+from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union, cast
 
 from graphql import (
     GraphQLArgument,
@@ -33,7 +25,7 @@ from graphql import (
 
 from strawberry.arguments import UNSET, StrawberryArgument, convert_arguments, is_unset
 from strawberry.custom_scalar import ScalarDefinition, ScalarWrapper
-from strawberry.directive import DirectiveDefinition
+from strawberry.directive import StrawberryDirective
 from strawberry.enum import EnumDefinition, EnumValue
 from strawberry.exceptions import (
     MissingTypesForGenericError,
@@ -41,7 +33,6 @@ from strawberry.exceptions import (
 )
 from strawberry.field import StrawberryField
 from strawberry.lazy_type import LazyType
-from strawberry.scalars import is_scalar
 from strawberry.schema.config import StrawberryConfig
 from strawberry.schema.types.scalar import _make_scalar_type
 from strawberry.type import StrawberryList, StrawberryOptional, StrawberryType
@@ -50,6 +41,7 @@ from strawberry.types.types import TypeDefinition
 from strawberry.union import StrawberryUnion
 from strawberry.utils.await_maybe import await_maybe
 
+from . import compat
 from .types.concrete_type import ConcreteType
 
 
@@ -93,22 +85,23 @@ class GraphQLCoreConverter:
         )
 
     def from_enum(self, enum: EnumDefinition) -> CustomGraphQLEnumType:
+        enum_name = self.config.name_converter.from_type(enum)
 
-        assert enum.name is not None
+        assert enum_name is not None
 
         # Don't reevaluate known types
-        if enum.name in self.type_map:
-            graphql_enum = self.type_map[enum.name].implementation
+        if enum_name in self.type_map:
+            graphql_enum = self.type_map[enum_name].implementation
             assert isinstance(graphql_enum, CustomGraphQLEnumType)  # For mypy
             return graphql_enum
 
         graphql_enum = CustomGraphQLEnumType(
-            name=enum.name,
+            name=enum_name,
             values={item.name: self.from_enum_value(item) for item in enum.values},
             description=enum.description,
         )
 
-        self.type_map[enum.name] = ConcreteType(
+        self.type_map[enum_name] = ConcreteType(
             definition=enum, implementation=graphql_enum
         )
 
@@ -117,15 +110,17 @@ class GraphQLCoreConverter:
     def from_enum_value(self, enum_value: EnumValue) -> GraphQLEnumValue:
         return GraphQLEnumValue(enum_value.value)
 
-    def from_directive(self, directive: DirectiveDefinition) -> GraphQLDirective:
-
+    def from_directive(self, directive: StrawberryDirective) -> GraphQLDirective:
         graphql_arguments = {}
+
         for argument in directive.arguments:
-            argument_name = argument.get_graphql_name(self.config.auto_camel_case)
+            argument_name = self.config.name_converter.from_argument(argument)
             graphql_arguments[argument_name] = self.from_argument(argument)
 
+        directive_name = self.config.name_converter.from_type(directive)
+
         return GraphQLDirective(
-            name=directive.name,
+            name=directive_name,
             locations=directive.locations,
             args=graphql_arguments,
             description=directive.description,
@@ -148,7 +143,7 @@ class GraphQLCoreConverter:
 
         graphql_arguments = {}
         for argument in field.arguments:
-            argument_name = argument.get_graphql_name(self.config.auto_camel_case)
+            argument_name = self.config.name_converter.from_argument(argument)
             graphql_arguments[argument_name] = self.from_argument(argument)
 
         return GraphQLField(
@@ -185,28 +180,30 @@ class GraphQLCoreConverter:
     def from_input_object(self, object_type: type) -> GraphQLInputObjectType:
         type_definition = object_type._type_definition  # type: ignore
 
+        type_name = self.config.name_converter.from_type(type_definition)
+
         # Don't reevaluate known types
-        if type_definition.name in self.type_map:
-            graphql_object_type = self.type_map[type_definition.name].implementation
+        if type_name in self.type_map:
+            graphql_object_type = self.type_map[type_name].implementation
             assert isinstance(graphql_object_type, GraphQLInputObjectType)  # For mypy
             return graphql_object_type
 
         def get_graphql_fields() -> Dict[str, GraphQLInputField]:
             graphql_fields = {}
             for field in type_definition.fields:
-                field_name = field.get_graphql_name(self.config.auto_camel_case)
+                field_name = self.config.name_converter.from_field(field)
 
                 graphql_fields[field_name] = self.from_input_field(field)
 
             return graphql_fields
 
         graphql_object_type = GraphQLInputObjectType(
-            name=type_definition.name,
+            name=type_name,
             fields=get_graphql_fields,
             description=type_definition.description,
         )
 
-        self.type_map[type_definition.name] = ConcreteType(
+        self.type_map[type_name] = ConcreteType(
             definition=type_definition, implementation=graphql_object_type
         )
 
@@ -215,9 +212,11 @@ class GraphQLCoreConverter:
     def from_interface(self, interface: TypeDefinition) -> GraphQLInterfaceType:
         # TODO: Use StrawberryInterface when it's implemented in another PR
 
+        interface_name = self.config.name_converter.from_type(interface)
+
         # Don't reevaluate known types
-        if interface.name in self.type_map:
-            graphql_interface = self.type_map[interface.name].implementation
+        if interface_name in self.type_map:
+            graphql_interface = self.type_map[interface_name].implementation
             assert isinstance(graphql_interface, GraphQLInterfaceType)  # For mypy
             return graphql_interface
 
@@ -225,35 +224,19 @@ class GraphQLCoreConverter:
             graphql_fields = {}
 
             for field in interface.fields:
-                field_name = field.get_graphql_name(self.config.auto_camel_case)
+                field_name = self.config.name_converter.from_field(field)
                 graphql_fields[field_name] = self.from_field(field)
 
             return graphql_fields
 
-        def resolve_type(
-            obj: Any,
-            info: GraphQLResolveInfo,
-            type_: Union[GraphQLInterfaceType, GraphQLUnionType],
-        ) -> GraphQLObjectType:
-            # TODO: this will probably break when passing dicts
-            # or even non strawberry types
-            resolved_type = self.type_map[
-                obj.__class__._type_definition.name
-            ].implementation
-
-            assert isinstance(resolved_type, GraphQLObjectType)
-
-            return resolved_type
-
         graphql_interface = GraphQLInterfaceType(
-            name=interface.name,
+            name=interface_name,
             fields=get_graphql_fields,
             interfaces=list(map(self.from_interface, interface.interfaces)),
             description=interface.description,
-            resolve_type=resolve_type,
         )
 
-        self.type_map[interface.name] = ConcreteType(
+        self.type_map[interface_name] = ConcreteType(
             definition=interface, implementation=graphql_interface
         )
 
@@ -278,10 +261,11 @@ class GraphQLCoreConverter:
 
     def from_object(self, object_type: TypeDefinition) -> GraphQLObjectType:
         # TODO: Use StrawberryObjectType when it's implemented in another PR
+        object_type_name = self.config.name_converter.from_type(object_type)
 
         # Don't reevaluate known types
-        if object_type.name in self.type_map:
-            graphql_object_type = self.type_map[object_type.name].implementation
+        if object_type_name in self.type_map:
+            graphql_object_type = self.type_map[object_type_name].implementation
             assert isinstance(graphql_object_type, GraphQLObjectType)  # For mypy
             return graphql_object_type
 
@@ -289,20 +273,32 @@ class GraphQLCoreConverter:
             graphql_fields = {}
 
             for field in object_type.fields:
-                field_name = field.get_graphql_name(self.config.auto_camel_case)
+                field_name = self.config.name_converter.from_field(field)
 
                 graphql_fields[field_name] = self.from_field(field)
 
             return graphql_fields
 
+        is_type_of: Optional[Callable[[Any, GraphQLResolveInfo], bool]]
+        if object_type.is_type_of:
+            is_type_of = object_type.is_type_of
+        elif object_type.interfaces:
+
+            def is_type_of(obj: Any, _info: GraphQLResolveInfo) -> bool:
+                return isinstance(obj, object_type.origin)
+
+        else:
+            is_type_of = None
+
         graphql_object_type = GraphQLObjectType(
-            name=object_type.name,
+            name=object_type_name,
             fields=get_graphql_fields,
             interfaces=list(map(self.from_interface, object_type.interfaces)),
             description=object_type.description,
+            is_type_of=is_type_of,
         )
 
-        self.type_map[object_type.name] = ConcreteType(
+        self.type_map[object_type_name] = ConcreteType(
             definition=object_type, implementation=graphql_object_type
         )
 
@@ -317,7 +313,10 @@ class GraphQLCoreConverter:
             kwargs: Dict[str, Any],
         ) -> Tuple[List[Any], Dict[str, Any]]:
             kwargs = convert_arguments(
-                kwargs, field.arguments, auto_camel_case=self.config.auto_camel_case
+                kwargs,
+                field.arguments,
+                scalar_registry=self.scalar_registry,
+                config=self.config,
             )
 
             # the following code allows to omit info and root arguments
@@ -413,58 +412,66 @@ class GraphQLCoreConverter:
         else:
             scalar_definition = scalar._scalar_definition
 
-        if scalar_definition.name not in self.type_map:
+        scalar_name = self.config.name_converter.from_type(scalar_definition)
+
+        if scalar_name not in self.type_map:
             implementation = (
                 scalar_definition.implementation
                 if scalar_definition.implementation is not None
                 else _make_scalar_type(scalar_definition)
             )
 
-            self.type_map[scalar_definition.name] = ConcreteType(
+            self.type_map[scalar_name] = ConcreteType(
                 definition=scalar_definition, implementation=implementation
             )
         else:
-            if self.type_map[scalar_definition.name].definition != scalar_definition:
-                raise ScalarAlreadyRegisteredError(scalar_definition.name)
+            if self.type_map[scalar_name].definition != scalar_definition:
+                raise ScalarAlreadyRegisteredError(scalar_name)
 
             implementation = cast(
-                GraphQLScalarType, self.type_map[scalar_definition.name].implementation
+                GraphQLScalarType, self.type_map[scalar_name].implementation
             )
 
         return implementation
 
     def from_type(self, type_: Union[StrawberryType, type]) -> GraphQLNullableType:
-        if _is_generic(type_):
+        if compat.is_generic(type_):
             raise MissingTypesForGenericError(type_)
 
         if isinstance(type_, EnumDefinition):  # TODO: Replace with StrawberryEnum
             return self.from_enum(type_)
-        elif _is_input_type(type_):  # TODO: Replace with StrawberryInputObject
+        elif compat.is_input_type(type_):  # TODO: Replace with StrawberryInputObject
             return self.from_input_object(type_)
         elif isinstance(type_, StrawberryList):
             return self.from_list(type_)
-        elif _is_interface_type(type_):  # TODO: Replace with StrawberryInterface
+        elif compat.is_interface_type(type_):  # TODO: Replace with StrawberryInterface
             type_definition: TypeDefinition = type_._type_definition  # type: ignore
             return self.from_interface(type_definition)
-        elif _is_object_type(type_):  # TODO: Replace with StrawberryObject
+        elif compat.is_object_type(type_):  # TODO: Replace with StrawberryObject
             type_definition: TypeDefinition = type_._type_definition  # type: ignore
             return self.from_object(type_definition)
+        elif compat.is_enum(type_):  # TODO: Replace with StrawberryEnum
+            enum_definition: EnumDefinition = type_._enum_definition  # type: ignore
+            return self.from_enum(enum_definition)
         elif isinstance(type_, TypeDefinition):  # TODO: Replace with StrawberryObject
             return self.from_object(type_)
-        elif _is_scalar(type_):  # TODO: Replace with StrawberryScalar
-            return self.from_scalar(type_)
         elif isinstance(type_, StrawberryUnion):
             return self.from_union(type_)
         elif isinstance(type_, LazyType):
             return self.from_type(type_.resolve_type())
+        elif compat.is_scalar(
+            type_, self.scalar_registry
+        ):  # TODO: Replace with StrawberryScalar
+            return self.from_scalar(type_)
 
         raise TypeError(f"Unexpected type '{type_}'")
 
     def from_union(self, union: StrawberryUnion) -> GraphQLUnionType:
+        union_name = self.config.name_converter.from_type(union)
 
         # Don't reevaluate known types
-        if union.name in self.type_map:
-            graphql_union = self.type_map[union.name].implementation
+        if union_name in self.type_map:
+            graphql_union = self.type_map[union_name].implementation
             assert isinstance(graphql_union, GraphQLUnionType)  # For mypy
             return graphql_union
 
@@ -477,57 +484,14 @@ class GraphQLCoreConverter:
             graphql_types.append(graphql_type)
 
         graphql_union = GraphQLUnionType(
-            name=union.name,
+            name=union_name,
             types=graphql_types,
             description=union.description,
             resolve_type=union.get_type_resolver(self.type_map),
         )
 
-        self.type_map[union.name] = ConcreteType(
+        self.type_map[union_name] = ConcreteType(
             definition=union, implementation=graphql_union
         )
 
         return graphql_union
-
-
-################################################################################
-# Temporary functions to be removed with new types
-################################################################################
-
-
-def _is_input_type(type_: Union[StrawberryType, type]) -> TypeGuard[type]:
-    if not _is_object_type(type_):
-        return False
-
-    type_definition: TypeDefinition = type_._type_definition  # type: ignore
-    return type_definition.is_input
-
-
-def _is_interface_type(type_: Union[StrawberryType, type]) -> TypeGuard[type]:
-    if not _is_object_type(type_):
-        return False
-
-    type_definition: TypeDefinition = type_._type_definition  # type: ignore
-    return type_definition.is_interface
-
-
-def _is_scalar(type_: Union[StrawberryType, type]) -> TypeGuard[type]:
-    # isinstance(type_, StrawberryScalar)  # noqa: E800
-    return is_scalar(type_)
-
-
-def _is_object_type(type_: Union[StrawberryType, type]) -> TypeGuard[type]:
-    # isinstance(type_, StrawberryObjectType)  # noqa: E800
-    return hasattr(type_, "_type_definition")
-
-
-def _is_generic(type_: Union[StrawberryType, type]) -> bool:
-    if hasattr(type_, "_type_definition"):
-
-        type_definition: TypeDefinition = type_._type_definition  # type: ignore
-        return type_definition.is_generic
-
-    if isinstance(type_, StrawberryType):
-        return type_.is_generic
-
-    return False
